ManualWipers = {}



function ManualWipers.prerequisitesPresent(specializations)

	return SpecializationUtil.hasSpecialization(Wipers, specializations)

end



function ManualWipers.registerEventListeners(vehicleType)

	local specFunctions = {	"onLoad", "saveToXMLFile", "onRegisterActionEvents", "onReadStream", "onWriteStream", "onReadUpdateStream", "onWriteUpdateStream" }
	
	for _, specFunction in ipairs(specFunctions) do
		SpecializationUtil.registerEventListener(vehicleType, specFunction, ManualWipers)
	end

end



function ManualWipers:onLoad(savegame)

	if self.spec_ManualWipers == nil then
		self.spec_ManualWipers = {}
	end

	local spec = self.spec_ManualWipers
	spec.actionEvents = {}
	spec.activated = false

	Wipers.onUpdateTick = Utils.overwrittenFunction(Wipers.onUpdateTick, ManualWipers.onUpdateTick)

	if savegame ~= nil then

		local Zustand = getXMLBool(savegame.xmlFile, savegame.key..".ManualWipers#manual")

		if Zustand ~= nil then
			spec.activated = Zustand
		end

	end

end



function ManualWipers:saveToXMLFile(xmlFile, key)

	local spec = self.spec_ManualWipers

	setXMLBool(xmlFile, key.."#manual", spec.activated)

end



function ManualWipers:onRegisterActionEvents(isActiveForInput)

	if self.isClient then

		local spec = self.spec_ManualWipers

		self:clearActionEventsTable(spec.actionEvents)
		spec.actionEventIds = {}

		if self:getIsActiveForInput(true, true) then

			local _, actionEventId = self:addActionEvent(spec.actionEvents, "CHI_ScheibenwischerEinAus", self, ManualWipers.onInputAction, false, true, false, true, nil)

			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_LOW)
			g_inputBinding:setActionEventTextVisibility(actionEventId, false)

			table.insert(spec.actionEventIds, actionEventId)

		end

	end

end



function ManualWipers:onReadStream(streamId, connection)

	self.spec_ManualWipers.activated = streamReadBool(streamId)

end



function ManualWipers:onWriteStream(streamId, connection)

	streamWriteBool(streamId, self.spec_ManualWipers.activated)

end



function ManualWipers:onReadUpdateStream(streamId, timestamp, connection)

	if connection.isServer then

		self.spec_ManualWipers.activated = streamReadBool(streamId)

	end

end



function ManualWipers:onWriteUpdateStream(streamId, connection, dirtyMask)

	if not connection.isServer then

		streamWriteBool(streamId, self.spec_ManualWipers.activated)

	end

end



function ManualWipers.onInputAction(self, actionName, inputValue, callbackState, isAnalog)

	local spec = self.spec_ManualWipers

	if actionName == "CHI_ScheibenwischerEinAus" then

		spec.activated = not spec.activated

	end

	if g_server ~= nil then

		g_server:broadcastEvent(ManualWipersEvent:new(self, spec.activated), nil, nil, self)

	else

		g_client:getServerConnection():sendEvent(ManualWipersEvent:new(self, spec.activated))

	end

end



function ManualWipers:onUpdateTick(superFunc, dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)

	local specWipers = self.spec_wipers
	local specAutoWipers = self.spec_ToggleableWiperautomatic
	local specManWipers = self.spec_ManualWipers

	local hasAutoWipers = specAutoWipers ~= nil
	local hasManWipers = specManWipers ~= nil

	specWipers.lastRainScale = g_currentMission.environment.weather:getRainFallScale()

	if specWipers.hasWipers then

		if hasManWipers then

			if specManWipers.activated and self:getIsMotorStarted() then

				for _, wiper in pairs(specWipers.wipers) do

					local stateIdToUse = #wiper.states

					if Wipers.forcedState ~= 0 then

						stateIdToUse = MathUtil.clamp(Wipers.forcedState, 1, #wiper.states)

					end

					if stateIdToUse > 0 then

						local currentState = wiper.states[stateIdToUse]

						if self:getAnimationTime(wiper.animName) == 1 then

							self:playAnimation(wiper.animName, -currentState.animSpeed, 1, true)

						end

						if wiper.nextStartTime == nil or wiper.nextStartTime < g_currentMission.time then

							if not self:getIsAnimationPlaying(wiper.animName) then

								self:playAnimation(wiper.animName, currentState.animSpeed, 0, true)
								wiper.nextStartTime = nil

							end

						end

						if wiper.nextStartTime == nil then

							wiper.nextStartTime = g_currentMission.time + (wiper.animDuration / currentState.animSpeed) + currentState.animPause

						end

					end

				end

			elseif hasAutoWipers then

				for _, wiper in pairs(specWipers.wipers) do

					local stateIdToUse = 0

					if self:getIsActiveForWipers() then

						if specWipers.lastRainScale > 0 then

							for stateIndex,state in ipairs(wiper.states) do

								if specWipers.lastRainScale <= state.maxRainValue then

									stateIdToUse = stateIndex
									break

								end

							end

						end

					end

					if Wipers.forcedState ~= 0 then

						stateIdToUse = MathUtil.clamp(Wipers.forcedState, 1, #wiper.states)

					end

					if stateIdToUse > 0 then

						local currentState = wiper.states[stateIdToUse]

						if self:getAnimationTime(wiper.animName) == 1 then

							self:playAnimation(wiper.animName, -currentState.animSpeed, 1, true)

						end

						if wiper.nextStartTime == nil or wiper.nextStartTime < g_currentMission.time then

							if not self:getIsAnimationPlaying(wiper.animName) then

								self:playAnimation(wiper.animName, currentState.animSpeed, 0, true)
								wiper.nextStartTime = nil

							end

						end

						if wiper.nextStartTime == nil then

							wiper.nextStartTime = g_currentMission.time + (wiper.animDuration / currentState.animSpeed) + currentState.animPause

						end

					end

				end

			end

		else

			if self:getIsControlled() then

				for _, wiper in pairs(specWipers.wipers) do

					local stateIdToUse = 0

					if self:getIsActiveForWipers() then

						if specWipers.lastRainScale > 0 then

							for stateIndex,state in ipairs(wiper.states) do

								if specWipers.lastRainScale <= state.maxRainValue then

									stateIdToUse = stateIndex
									break

								end

							end

						end

					end

					if Wipers.forcedState ~= 0 then

						stateIdToUse = MathUtil.clamp(Wipers.forcedState, 1, #wiper.states)

					end

					if stateIdToUse > 0 then

						local currentState = wiper.states[stateIdToUse]

						if self:getAnimationTime(wiper.animName) == 1 then

							self:playAnimation(wiper.animName, -currentState.animSpeed, 1, true)

						end

						if wiper.nextStartTime == nil or wiper.nextStartTime < g_currentMission.time then

							if not self:getIsAnimationPlaying(wiper.animName) then

								self:playAnimation(wiper.animName, currentState.animSpeed, 0, true)
								wiper.nextStartTime = nil

							end

						end

						if wiper.nextStartTime == nil then

							wiper.nextStartTime = g_currentMission.time + (wiper.animDuration / currentState.animSpeed) + currentState.animPause

						end

					end

				end

			end

		end

	end

end





-- Events

ManualWipersEvent = {}
ManualWipersEvent_mt = Class(ManualWipersEvent, Event)

InitEventClass(ManualWipersEvent, "ManualWipersEvent")



function ManualWipersEvent:emptyNew()

	local self = Event:new(ManualWipersEvent_mt)

	self.className="ManualWipersEvent"

	return self

end



function ManualWipersEvent:new(vehicle, activated)

	local self = ManualWipersEvent:emptyNew()

	self.vehicle = vehicle
	self.activated = activated

	return self

end



function ManualWipersEvent:readStream(streamId, connection)

	self.vehicle = NetworkUtil.readNodeObject(streamId)
	self.activated = streamReadBool(streamId)

	self:run(connection)

end



function ManualWipersEvent:writeStream(streamId, connection)

	NetworkUtil.writeNodeObject(streamId, self.vehicle)

	streamWriteBool(streamId, self.activated)

end



function ManualWipersEvent:run(connection)

	self.vehicle.spec_ManualWipers.activated = self.activated

	if self.vehicle ~= nil then

		if not connection:getIsServer() then

			g_server:broadcastEvent(ManualWipersEvent:new(self.vehicle, self.activated), nil, connection, self.vehicle)

		end

	end

end



function ManualWipersEvent.sendEvent(vehicle, activated, noEventSend)

	if noEventSend == nil or noEventSend == false then

		if g_server ~= nil then

			g_server:broadcastEvent(ManualWipersEvent:new(vehicle, activated), nil, nil, vehicle)

		else

			g_client:getServerConnection():sendEvent(ManualWipersEvent:new(vehicle, activated))

		end

	end

end